#!/usr/bin/python
#identitypropagate.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2013
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#
# When a user starts up he needs to run the stun.py to check what his IP is,
#  and if it has changed he needs to generate a new identity and send it to
#  his identityserver and all of his contacts.
#
# We also just request new copies of all identities from their servers when
#   we start up.   This is simple and effective.
#
# We should try contacting each contact every hour and if we have not been
# able to contact them in 2 or 3 hours then fetch copy of identity from
# their server.   PREPRO


import os
import sys


try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in identitypropagate.py')


from twisted.internet.defer import DeferredList, Deferred


import lib.dhnio as dhnio
import lib.misc as misc
import lib.nameurl as nameurl
import lib.dhnpacket as dhnpacket
import lib.identitycache as identitycache
import lib.contacts as contacts
import lib.commands as commands
import lib.settings as settings
import lib.stun as stun
import lib.tmpfile as tmpfile
import lib.transport_control as transport_control
import lib.transport_tcp as transport_tcp


_SlowSendIsWorking = False

#------------------------------------------------------------------------------


def init():
    dhnio.Dprint(4, "identitypropagate.init ")
    transport_control.SetPingContactFunc(PingContact)


def propagate(selected_contacts, AckHandler=None, wide=False):
    dhnio.Dprint(6, "identitypropagate.propagate to %d contacts" % len(selected_contacts))
    d = Deferred()
    def contacts_fetched(x, d, selected_contacts):
        dhnio.Dprint(6, "identitypropagate.propagate.contacts_fetched")
        SendToIDs(selected_contacts, AckHandler, wide)
        d.callback('sent')
        return x
    fetch(selected_contacts).addBoth(contacts_fetched, d, selected_contacts)
    return d


def fetch(list_ids): 
    dhnio.Dprint(6, "identitypropagate.fetch identities for %d users" % len(list_ids))
    dl = []
    for url in list_ids:
        if url:
            if not identitycache.FromCache(url):
                dl.append(identitycache.scheduleForCaching(url))
    return DeferredList(dl, consumeErrors=True)


def start(AckHandler=None, wide=False):
    return propagate(contacts.getRemoteContacts(), AckHandler, wide)


def suppliers(AckHandler=None, wide=False):
    return propagate(contacts.getSupplierIDs(), AckHandler, wide)


def customers(AckHandler=None, wide=False):
    return propagate(contacts.getCustomerIDs(), AckHandler, wide)


def single(idurl, AckHandler=None, wide=False):
    FetchSingle(idurl).addBoth(lambda x: SendToIDs([idurl], AckHandler, wide))
    

def update():
    dhnio.Dprint(6, "identitypropagate.update")
    return SendServers()


def FetchSingle(idurl):
    dhnio.Dprint(6, "identitypropagate.fetch_single " + idurl)
    return identitycache.scheduleForCaching(idurl)


def Fetch(idslist):
    return fetch(idslist)


def FetchSuppliers():
    return fetch(contacts.getSupplierIDs())


def FetchCustomers():
    return fetch(contacts.getCustomerIDs())


def FetchNecesaryContacts():
    return fetch(contacts.getContactIDs())
def FetchAllContacts(): 
    return fetch(contacts.getContactIDs())


def SendServers():
    sendfile, sendfilename = tmpfile.make("propagate")
    os.close(sendfile)
    LocalIdentity = misc.getLocalIdentity()
    dhnio.WriteFile(sendfilename, LocalIdentity.serialize())
    dlist = []
    for idurl in LocalIdentity.sources:
        # sources for out identity are servers we need to send to
        protocol, host, port, filename = nameurl.UrlParse(idurl)
        port = settings.IdentityServerPort()
        d = Deferred()
        transport_tcp.send(sendfilename, host, port, do_status_report=False, result_defer=d, description='Identity')
        dlist.append(d) 
    dl = DeferredList(dlist, consumeErrors=True)
    return dl


def SendSingleSupplier(idurl, response_callback=None):
    dhnio.Dprint(6, "identitypropagate.SendSingleSupplier [%s]" % nameurl.GetName(idurl))
    MyID = misc.getLocalID()
    packet = dhnpacket.dhnpacket(commands.Identity(), MyID, MyID, MyID, misc.getLocalIdentity().serialize(), idurl)
    transport_control.outboxNoAck(packet)
    if response_callback is not None:
        transport_control.RegisterInterest(response_callback, packet.RemoteID, packet.PacketID)


def SendSingleCustomer(idurl, response_callback=None):
    dhnio.Dprint(6, "identitypropagate.SendSingleCustomer [%s]" % nameurl.GetName(idurl))
    MyID = misc.getLocalID()
    packet = dhnpacket.dhnpacket(commands.Identity(), MyID, MyID, MyID, misc.getLocalIdentity().serialize(), idurl)
    transport_control.outboxNoAck(packet)
    if response_callback is not None:
        transport_control.RegisterInterest(response_callback, packet.RemoteID, packet.PacketID)


def SendContacts():
    dhnio.Dprint(6, "identitypropagate.SendContacts")
    SendToIDs(contacts.getContactIDs(), HandleAck)


def SendSuppliers():
    dhnio.Dprint(6, "identitypropagate.SendSuppliers")
#    guistatus.InitCallSuppliers()
    RealSendSuppliers()


def RealSendSuppliers():
    dhnio.Dprint(8, "identitypropagate.RealSendSuppliers")
    SendToIDs(contacts.getSupplierIDs(), HandleSuppliersAck)


def SlowSendSuppliers(delay=1):
    global _SlowSendIsWorking
    if _SlowSendIsWorking:
        dhnio.Dprint(8, "identitypropagate.SlowSendSuppliers  is working at the moment. skip.")
        return
    dhnio.Dprint(8, "identitypropagate.SlowSendSuppliers delay=%s" % str(delay))

    def _send(index, payload, delay):
        global _SlowSendIsWorking
        idurl = contacts.getSupplierID(index)
        if not idurl:
            _SlowSendIsWorking = False
            return
        transport_control.ClearAliveTime(idurl)
        SendToID(idurl, Payload=payload, wide=True)
        reactor.callLater(delay, _send, index+1, payload, delay)

    _SlowSendIsWorking = True
    payload = misc.getLocalIdentity().serialize()
    _send(0, payload, delay)


def SlowSendCustomers(delay=1):
    global _SlowSendIsWorking
    if _SlowSendIsWorking:
        dhnio.Dprint(8, "identitypropagate.SlowSendCustomers  slow send is working at the moment. skip.")
        return
    dhnio.Dprint(8, "identitypropagate.SlowSendCustomers delay=%s" % str(delay))

    def _send(index, payload, delay):
        global _SlowSendIsWorking
        idurl = contacts.getCustomerID(index)
        if not idurl:
            _SlowSendIsWorking = False
            return
        transport_control.ClearAliveTime(idurl)
        SendToID(idurl, Payload=payload, wide=True)
        reactor.callLater(delay, _send, index+1, payload, delay)

    _SlowSendIsWorking = True
    payload = misc.getLocalIdentity().serialize()
    _send(0, payload, delay)

def SendCustomers():
    dhnio.Dprint(8, "identitypropagate.SendCustomers")
#    guistatus.InitCallCustomers()
    RealSendCustomers()


def RealSendCustomers():
    dhnio.Dprint(8, "identitypropagate.RealSendCustomers")
    SendToIDs(contacts.getCustomerIDs(), HandleCustomersAck)


def HandleSingleSupplier(ackpacket):
    Num = contacts.numberForSupplier(ackpacket.OwnerID)
#    guistatus.SetShortStatusAlive(ackpacket, Num, "suppliers")


def HandleSingleCustomer(ackpacket):
    Num = contacts.numberForCustomer(ackpacket.OwnerID)
#    guistatus.SetShortStatusAlive(ackpacket, Num, "customers")


def HandleAck(ackpacket):
    #Num = contacts.numberForContact(ackpacket.OwnerID)
    dhnio.Dprint(6, "identitypropagate.HandleAck " + nameurl.GetName(ackpacket.OwnerID))
#    guistatus.SetShortStatusAlive(ackpacket, Num, "contacts")


def HandleSuppliersAck(ackpacket):
    Num = contacts.numberForSupplier(ackpacket.OwnerID)
    dhnio.Dprint(8, "identitypropagate.HandleSupplierAck ")
#    guistatus.SetShortStatusAlive(ackpacket, Num, "suppliers")


def HandleCustomersAck(ackpacket):
    Num = contacts.numberForCustomer(ackpacket.OwnerID)
    dhnio.Dprint(8, "identitypropagate.HandleCustomerAck ")
#    guistatus.SetShortStatusAlive(ackpacket, Num, "customers")


def SendToID(idurl, AckHandler=None, Payload=None, NeedAck=False, wide=False):
    dhnio.Dprint(8, "identitypropagate.SendToID [%s] NeedAck=%s" % (nameurl.GetName(idurl), str(NeedAck)))
    thePayload = Payload
    if thePayload is None:
        thePayload = misc.getLocalIdentity().serialize()
    packet = dhnpacket.dhnpacket(
        commands.Identity(),
        misc.getLocalID(), #MyID,
        misc.getLocalID(), #MyID,
        misc.getLocalID(), #PacketID,
        thePayload,
        idurl)
    if AckHandler is not None:
        transport_control.RegisterInterest(AckHandler, packet.RemoteID, packet.PacketID)
    transport_control.outbox(packet, NeedAck, wide)
    if wide:
        # this is a ping packet - need to clear old info
        transport_control.ErasePeerProtosStates(idurl)
        transport_control.EraseMyProtosStates(idurl)


def SendToIDs(idlist, AckHandler=None, wide=False):
    dhnio.Dprint(6, "identitypropagate.SendToIDs to %d users" % len(idlist))
    MyID = misc.getLocalID()
    PacketID = MyID
    LocalIdentity = misc.getLocalIdentity()
    Payload = LocalIdentity.serialize()
    alreadysent = set()
    for contact in idlist:
        if not contact:
            continue
        if contact in alreadysent:
            # just want to send once even if both customer and supplier
            continue
        packet = dhnpacket.dhnpacket(
            commands.Identity(),
            misc.getLocalID(), #MyID,
            misc.getLocalID(), #MyID,
            misc.getLocalID(), #PacketID,
            Payload,
            contact)
        dhnio.Dprint(8, "identitypropagate.SendToIDs to " + contact)
        if AckHandler is not None:
            transport_control.RegisterInterest(AckHandler, packet.RemoteID, packet.PacketID)
        transport_control.outboxNoAck(packet, wide)
        if wide:
            # this is a ping packet - need to clear old info
            transport_control.ErasePeerProtosStates(contact)
            transport_control.EraseMyProtosStates(contact)
        alreadysent.add(contact)
    del alreadysent


def PingContact(idurl):
    SendToID(idurl, NeedAck=True)





#!/usr/bin/python
#raidmake.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2013
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#

#
# Input is a file and a RAID spec.
# The spec says for each datablock which parity blocks XOR it.
# As we read in each datablock we XOR all the parity blocks for it.
# Probably best to read in all the datablocks at once, and do all
# the parities one time.  If we did one datablock after another
# the parity blocks would need to be active all the time.
#
# If we XOR integers, we have a byte order issue probably, though
# maybe not since they all stay the same order, whatever that is.
#
# Some machines are 64-bit.  Would be fun to make use of that if
# we have it.
#
# 2^1 => 3       ^ is XOR operator
#
# Parity packets have to be a little longer so they can hold
# parities on signatures of datapackets.  So we can reconstruct
# those signatures.
#
#
import os
import sys
import struct
import time
import cStringIO


if __name__ == '__main__':
    dirpath = os.path.dirname(os.path.abspath(sys.argv[0]))
    sys.path.insert(0, os.path.abspath(os.path.join(dirpath, '..', '..')))


import lib.eccmap as eccmap
import lib.misc as misc
import lib.dhnio as dhnio
import lib.settings as settings
import lib.tmpfile as tmpfile


#------------------------------------------------------------------------------ 

def raidmake(filename, eccmapname, backupId, blockNumber, data_parity_dir=None, in_memory=True):
    dhnio.Dprint(8, "raidmake.raidmake BEGIN %s %s %s %d" % (
        os.path.basename(filename), eccmapname, backupId, blockNumber))
    t = time.time()
    if in_memory:
        dataNum, parityNum, outDir = do_in_memory(filename, eccmapname, backupId, blockNumber, data_parity_dir)
    else:
        dataNum, parityNum, outDir = do_with_files(filename, eccmapname, backupId, blockNumber, data_parity_dir)
    dhnio.Dprint(8, "raidmake.raidmake END time=%.3f data=%d parity=%d" % (time.time()-t, dataNum, parityNum))
    return outDir


def do_in_memory(filename, eccmapname, backupId, blockNumber, data_parity_dir=None):
    if data_parity_dir is None:
        # data_parity_dir = tmpfile.subdir('data-par')
        data_parity_dir = settings.getLocalBackupsDir()

    myeccmap = eccmap.eccmap(eccmapname)
    INTSIZE = settings.IntSize()
    # any padding at end and block.Length fixes
    misc.RoundupFile(filename,myeccmap.datasegments*INTSIZE)     
    wholefile = dhnio.ReadBinaryFile(filename)
    length = len(wholefile)
    seglength = (length + myeccmap.datasegments - 1) / myeccmap.datasegments                 

    for DSegNum in range(myeccmap.datasegments):
        FileName = data_parity_dir + '/' + backupId + '-' + str(blockNumber) + '-' + str(DSegNum) + '-Data'
        #FileName = os.path.join(data_parity_dir, backupId + '-' + str(blockNumber) + '-' + str(DSegNum) + '-Data')
        f = open(FileName, "wb")
        segoffset = DSegNum * seglength
        for i in range(seglength):
            offset = segoffset + i;
            if offset < length:
                f.write(wholefile[offset])
            else:
                # any padding should go at the end of last seg 
                # and block.Length fixes
                f.write(" ")               
        f.close()
        
    dfds = {}
    for DSegNum in range(myeccmap.datasegments):
        FileName = data_parity_dir + '/' + backupId + '-' + str(blockNumber) + '-' + str(DSegNum) + '-Data'
        #FileName = os.path.join(data_parity_dir, backupId + '-' + str(blockNumber) + '-' + str(DSegNum) + '-Data')
        # instead of reading data from opened file 
        # we'l put it in memory 
        # and store current position in the data
        # so start from zero 
        #dfds[DSegNum] = [0, dhnio.ReadBinaryFile(FileName)]
        dfds[DSegNum] = cStringIO.StringIO(dhnio.ReadBinaryFile(FileName))

    pfds = {}
    for PSegNum in range(myeccmap.paritysegments):
        # we will keep parirty data in the momory
        # after doing all calculations
        # will write all parts on the disk
        pfds[PSegNum] = cStringIO.StringIO()

    #Parities = range(myeccmap.paritysegments)
    Parities = {}
    for i in range(seglength/INTSIZE):
        for PSegNum in range(myeccmap.paritysegments):
            Parities[PSegNum] = 0
        for DSegNum in range(myeccmap.datasegments):
            bstr = dfds[DSegNum].read(INTSIZE)
            #pos = dfds[DSegNum][0]
            #dfds[DSegNum][0] += INTSIZE
            #bstr = dfds[DSegNum][1][pos:pos+INTSIZE]
            if len(bstr) == INTSIZE:
                b, = struct.unpack(">l", bstr)
                Map = myeccmap.DataToParity[DSegNum]
                for PSegNum in Map:
                    if PSegNum > myeccmap.paritysegments:
                        dhnio.Dprint(2, "raidmake.raidmake PSegNum out of range " + str(PSegNum))
                        dhnio.Dprint(2, "raidmake.raidmake limit is " + str(myeccmap.paritysegments))
                        myeccmap.check()
                        raise Exception("eccmap error")
                    Parities[PSegNum] = Parities[PSegNum] ^ b
            else:
                raise Exception('strange read under INTSIZE bytes, len(bstr)=%d DSegNum=%d' % (len(bstr), DSegNum)) 
                #TODO
                #dhnio.Dprint(2, 'raidmake.raidmake WARNING strange read under INTSIZE bytes')
                #dhnio.Dprint(2, 'raidmake.raidmake len(bstr)=%s DSegNum=%s' % (str(len(bstr)), str(DSegNum)))

        for PSegNum in range(myeccmap.paritysegments):
            bstr = struct.pack(">l", Parities[PSegNum])
            #pfds[PSegNum] += bstr
            pfds[PSegNum].write(bstr)

    dataNum = len(dfds)
    parityNum = len(pfds)
    
    for PSegNum, data in pfds.items():
        #FileName = os.path.join(data_parity_dir, backupId + '-' + str(blockNumber) + '-' + str(PSegNum) + '-Parity')
        FileName = data_parity_dir + '/' + backupId + '-' + str(blockNumber) + '-' + str(PSegNum) + '-Parity'
        dhnio.WriteFile(FileName, pfds[PSegNum].getvalue())

    for f in dfds.values():
        f.close()
        #dataNum += 1

    for f in pfds.values():
        f.close()
        #parityNum += 1
    
    del dfds
    del pfds
    del Parities

    return dataNum, parityNum, data_parity_dir


def do_with_files(filename, eccmapname, backupId, blockNumber, data_parity_dir=None):
    if data_parity_dir is None:
        # data_parity_dir = tmpfile.subdir('data-par')
        data_parity_dir = settings.getLocalBackupsDir()

    myeccmap = eccmap.eccmap(eccmapname)
    INTSIZE = settings.IntSize()
    misc.RoundupFile(filename,myeccmap.datasegments*INTSIZE)      # any padding at end and block.Length fixes
    wholefile = dhnio.ReadBinaryFile(filename)
    length = len(wholefile)
    seglength = (length + myeccmap.datasegments - 1)/myeccmap.datasegments                 # PREPRO -

    for DSegNum in range(myeccmap.datasegments):
        FileName = data_parity_dir + '/' + backupId + '-' + str(blockNumber) + '-' + str(DSegNum) + '-Data'
        f = open(FileName, "wb")
        segoffset = DSegNum * seglength
        for i in range(seglength):
            offset = segoffset + i;
            if (offset < length):
                f.write(wholefile[offset])
            else:
                # any padding should go at the end of last seg 
                # and block.Length fixes
                f.write(" ")               
        f.close()
    del wholefile

    #dfds = range(myeccmap.datasegments)
    dfds = {}
    for DSegNum in range(myeccmap.datasegments):
        FileName = data_parity_dir + '/' + backupId + '-' + str(blockNumber) + '-' + str(DSegNum) + '-Data'
        dfds[DSegNum] = open(FileName, "rb")

    #pfds = range(myeccmap.paritysegments)
    pfds = {}
    for PSegNum in range(myeccmap.paritysegments):
        FileName = data_parity_dir + '/' + backupId + '-' + str(blockNumber) + '-' + str(PSegNum) + '-Parity'
        pfds[PSegNum] = open(FileName, "wb")

    #Parities = range(myeccmap.paritysegments)
    Parities = {}
    for i in range(seglength/INTSIZE):
        for PSegNum in range(myeccmap.paritysegments):
            Parities[PSegNum] = 0
        for DSegNum in range(myeccmap.datasegments):
            bstr = dfds[DSegNum].read(INTSIZE)
            if len(bstr) == INTSIZE:
                b, = struct.unpack(">l", bstr)
                Map = myeccmap.DataToParity[DSegNum]
                for PSegNum in Map:
                    if PSegNum > myeccmap.paritysegments:
                        dhnio.Dprint(2, "raidmake.raidmake PSegNum out of range " + str(PSegNum))
                        dhnio.Dprint(2, "raidmake.raidmake limit is " + str(myeccmap.paritysegments))
                        myeccmap.check()
                        raise Exception("eccmap error")
                    Parities[PSegNum] = Parities[PSegNum] ^ b
            else :
                #TODO
                dhnio.Dprint(2, 'raidmake.raidmake WARNING strange read under INTSIZE bytes')
                dhnio.Dprint(2, 'raidmake.raidmake len(bstr)=%s DSegNum=%s' % (str(len(bstr)), str(DSegNum)))

        for PSegNum in range(myeccmap.paritysegments):
            bstr = struct.pack(">l", Parities[PSegNum])
            pfds[PSegNum].write(bstr)

    dataNum = 0
    parityNum = 0
    
    for f in dfds.values():
        f.close()
        dataNum += 1

    for f in pfds.values():
        f.close()
        parityNum += 1

    del dfds
    del pfds
    del Parities

    return dataNum, parityNum, data_parity_dir



def main():
    dhnio.SetDebug(18)
    raidmake(sys.argv[1], sys.argv[2], sys.argv[3], int(sys.argv[4]), sys.argv[5], sys.argv[6]=='1')
    

if __name__ == "__main__":
    main()






import hashlib

class DigestError(Exception):
    def __init__( self, *args ) :
        Exception.__init__( self, *args )

class DigestType :
    def __init__( self, evpMd ) :
        self.m = hashlib.new(evpMd.lower())

    def name(self) :
        return self.m.name

    def size(self) :
        return self.m.digest_size

    def blockSize(self) :
        return self.m.block_size

class Digest:
    def __init__(self, digestType) :
        self.hash = digestType.m
        self.finalized = False

    def update(self, data) :
        if self.finalized:
            raise DigestError, 'digest operation is already completed'
        return self.hash.update(data)

    def digest(self, data=None) :
        if self.finalized:
            raise DigestError, 'digest operation is already completed'
        if data is not None :
            self.hash.update(data)   
        result = self.hash.digest()
        self.finalized = True      
        return result



#!/usr/bin/python
#dhnpacket.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2013
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
# These packets usually hold on the order of 1 MB.
# Something equal to a packet number so we can detect duplicates in transport
import os
import sys

from twisted.internet import threads
from twisted.internet.defer import Deferred


import types
import datetime


import dhnio
import commands
import misc
import dhncrypto
import packetid
import contacts


# Legal Commands are in commands.py
# Packet Fields are all strings (no integers, objects, etc):
class dhnpacket:
# PacketID - string of the above 4 "Number"s with "-" separator to uniquely identify a packet
# on the local machine.  Can be used for filenames, and to prevent duplicates.
    def __init__(self, Command, OwnerID, CreatorID, PacketID, Payload, RemoteID,):
        self.Command = Command               
        # who owns this data and pays bills - http://cate.com/id1.xml
        self.OwnerID = OwnerID                
        # signer - http://cate.com/id1.xml - might be an authorized scrubber
        self.CreatorID = CreatorID            
        # functions for making and reading parts of this
        self.PacketID = PacketID              
        self.Date = datetime.datetime.now().strftime("%Y/%m/%d %I:%M:%S %p")
        # main body of binary data
        self.Payload = Payload                
        # want full IDURL for other party so troublemaker could not
        # use his packets to mess up other nodes by sending it to them
        self.RemoteID = RemoteID             
        # signature on Hash is always by CreatorID
        self.Signature = None
        self.Sign()

    def __repr__(self):
        return 'dhnpacket (command=%s id=%s)' % (str(self.Command), str(self.PacketID))

    def Sign(self):
        self.Signature = self.GenerateSignature()  # usually just done at packet creation
        return self

    def GenerateHashBase(self):
        sep = "-"
        stufftosum = self.Command + sep + self.OwnerID + sep + self.CreatorID + sep + self.PacketID + sep + self.Date + sep + self.Payload + sep + self.RemoteID
        return stufftosum

    def GenerateHash(self):
        return dhncrypto.Hash(self.GenerateHashBase())

    def GenerateSignature(self):
        return dhncrypto.Sign(self.GenerateHash())

    def SignatureChecksOut(self):
        ConIdentity = contacts.getContact(self.CreatorID)
        if ConIdentity is None:
            dhnio.Dprint(1, "dhnpacket.SignatureChecksOut ERROR could not get Identity for " + self.CreatorID + " so returning False")
            return False
        Result = dhncrypto.Verify(ConIdentity, self.GenerateHash(), self.Signature)
        return Result

    def Ready(self):
        return self.Signature is not None

    def Valid(self):
        # Valid should check every one of packet hearder fields:
        #         1) that command is one of the legal commands
        #         2) signature is good (which means the hashcode is good)
        # Rest PREPRO:
        #         2) all the number fields are just numbers
        #         5) length is within legal limits
        #         6) check that URL is a good URL
        #         7) that DataOrParity is either "data" or "parity"
        #         8) that creator is equal to owner or a scrubber for owner
        #         etc
        if not self.Ready():
            dhnio.Dprint(4, "dhnpacket.Valid WARNING packet is not ready yet " + str(self))
            return False
        if not commands.IsCommand(self.Command):
            dhnio.Dprint(1, "dhnpacket.Valid bad Command " + str(self.Command))
            return False
        if not self.SignatureChecksOut():
            dhnio.Dprint(1, "dhnpacket.Valid failed Signature")
            return False
        return True

    def BackupID(self):
        return packetid.BackupID(self.PacketID)

    def BlockNumber(self):
        return packetid.BlockNumber(self.PacketID)

    def DataOrParity(self):
        return packetid.DataOrParity(self.PacketID)

    def SupplierNumber(self):
        return packetid.SupplierNumber(self.PacketID)

    def Serialize(self):
        return misc.ObjectToString(self)

    def __len__(self):
        return len(self.Serialize())


class dhnpacket_0signed(dhnpacket):
    
    def GenerateSignature(self):
        return '0'


def Unserialize(data):
    if data is None:
        return None
    newobject = misc.StringToObject(data)
    if type(newobject) != types.InstanceType:
        dhnio.Dprint(6, "dhnpacket.Unserialize WARNING not an instance: " + str(newobject))
        return None
    # if newobject.__class__ != dhnpacket or str(newobject.__class__).count('dhnpacket') == 0:
    if not str(newobject.__class__).count('dhnpacket.dhnpacket'):
        dhnio.Dprint(6, "dhnpacket.Unserialize WARNING not a dhnpacket: " + str(newobject.__class__))
        return None
    return newobject

def MakePacket(Command, OwnerID, CreatorID, PacketID, Payload, RemoteID):
    result = dhnpacket(Command, OwnerID, CreatorID, PacketID, Payload, RemoteID)
    return result

def MakePacketInThread(CallBackFunc, Command, OwnerID, CreatorID, PacketID, Payload, RemoteID):
    d = threads.deferToThread(MakePacket, Command, OwnerID, CreatorID, PacketID, Payload, RemoteID)
    d.addCallback(CallBackFunc)

def MakePacketDeferred(Command, OwnerID, CreatorID, PacketID, Payload, RemoteID):
    return threads.deferToThread(MakePacket, Command, OwnerID, CreatorID, PacketID, Payload, RemoteID)

#------------------------------------------------------------------------------ 

if __name__ == '__main__':
    dhnio.init()
    dhnio.SetDebug(18)
    import settings
    settings.init()
    dhncrypto.InitMyKey()
    p = Unserialize(dhnio.ReadBinaryFile(sys.argv[1]))
    print p
    
    
    


import logging
from types import IntType, StringType
from socket import error as sock_error
from ncrypt0.rand import bytes as rand_bytes
from nitro.async import AsyncOp
from nitro.bencode import encode, decode, DecodeError

logger = logging.getLogger( 'cspace.dht.rpc' )

class RPCContext(object) :
    def __init__( self, id, addr, rpcSocket ) :
        self.id = id
        self.addr = addr
        self.rpcSocket = rpcSocket

    def response( self, data ) :
        self.rpcSocket.response( data, self )

class RPCOp( AsyncOp ) :
    def __init__( self, opId, callback=None, canceler=None ) :
        AsyncOp.__init__( self, callback, canceler )
        self.opId = opId

    def getId( self ) : return self.opId

class RPCSocket( object ) :
    def __init__( self, sock, reactor ) :
        self.sock = sock
        self.reactor = reactor
        self.pending = {}
        self.requestCallback = None
        reactor.addReadCallback( sock.fileno(), self._onRead )

    def getAddr( self ) : return self.sock.getsockname()
    
    def setRequestCallback( self, requestCallback ) :
        self.requestCallback = requestCallback

    def close( self ) :
        self.reactor.removeReadCallback( self.sock.fileno() )
        self.sock.close()

    def _onRead( self ) :
        try :
            (data,fromaddr) = self.sock.recvfrom( 8192 )
        except sock_error, (err,errMsg) :
            return
        self._onInput( data, fromaddr )

    def _onInput( self, data, fromaddr ) :
        try :
            (msgCode,id,payload) = decode( data )
            if (type(msgCode) is not IntType) or (type(id) is not StringType) :
                raise TypeError, 'invalid msg structure'
        except DecodeError, de :
            logger.exception( 'decode error: %s', de )
            return
        except (TypeError, ValueError), e :
            logger.exception( 'invalid msg: %s', e )
            return
        if msgCode == 0 : # incoming request
            ctx = RPCContext( id, fromaddr, self )
            self._onRequest( payload, ctx )
        elif msgCode == 1 : # incoming response
            op = self.pending.pop( id, None )
            if op is None :
                logger.warning( 'invalid response msg' )
                return
            op.notify( payload, fromaddr )
        else :
            logger.warning( 'invalid msg code: %d', msgCode )

    def _onRequest( self, payload, ctx ) :
        if self.requestCallback is not None :
            self.requestCallback( payload, ctx )
            return
        logger.warning( 'unhandled incoming msg from %s', ctx.addr )

    def _sendData( self, data, destaddr ) :
        try :
            self.sock.sendto( data, 0, destaddr )
        except sock_error, (err,errMsg) :
            logger.info( 'error sending data: %s' % str(errMsg))

    def request( self, data, destaddr, callback=None ) :
        while 1 :
            id = rand_bytes( 20 )
            if not self.pending.has_key(id) : break
        msg = (0, id, data)
        s = encode( msg )
        def cancelRequest() :
            del self.pending[id]
        op = RPCOp( id, callback, cancelRequest )
        self.pending[id] = op
        self._sendData( s, destaddr )
        return op

    def retry( self, id, data, destaddr ) :
        assert id in self.pending
        msg = (0, id, data)
        s = encode( msg )
        self._sendData( s, destaddr )

    def response( self, data, ctx ) :
        assert ctx.rpcSocket is self
        msg = (1, ctx.id, data )
        s = encode( msg )
        self._sendData( s, ctx.addr )

    def call( self, data, destaddr, callback=None,
            timeout=2, retries=1, backoff=1 ) :
        return _Call( data, destaddr, self, callback,
                timeout, retries, backoff ).getOp()

class _Call(object) :
    def __init__( self, data, destaddr, rpcSocket, callback, timeout, retries, backoff ) :
        self.data = data
        self.destaddr = destaddr
        self.rpcSocket = rpcSocket
        self.timeout = timeout
        self.retries = retries
        self.backoff = backoff
        self.reactor = rpcSocket.reactor
        self.attempt = 0
        self.rpcOp = rpcSocket.request( data, destaddr, self.onResult )
        self.timerOp = self.reactor.callLater( timeout, self.onTimeout )
        self.op = RPCOp( self.rpcOp.getId(), callback, self.cancel )
        self.active = True

    def getOp( self ) : return self.op

    def cancel( self ) :
        assert self.active
        self.active = False
        self.rpcOp.cancel()
        self.timerOp.cancel()

    def onResult( self, payload, fromaddr ) :
        assert self.active
        self.active = False
        self.timerOp.cancel()
        self.op.notify( self.attempt, payload )

    def onTimeout( self ) :
        assert self.active
        if self.attempt == self.retries :
            self.active = False
            self.rpcOp.cancel()
            self.op.notify( -1, None )
        else :
            self.attempt += 1
            self.timeout *= self.backoff
            self.timerOp = self.reactor.callLater( self.timeout, self.onTimeout )
            self.rpcSocket.retry( self.rpcOp.getId(), self.data, self.destaddr )



#!/usr/bin/python

#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2013
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#

#import time, sys, os, sched             # sched lets you call os to schedule future commands
#
#import dobackup
#                                        #  Google or Python Coockbook page 133

#class backupinfo:
#    frequency=""
#    fullinc=""
#    bieccmap=""
#    nextid=""
#    nexttime=""
#
#def backupmanager():
#    dobackup.dobackup()                 # right now we just do one backup - not so clever at the moment








import os, sys, logging
from types import StringType, ListType

logger = logging.getLogger( 'cspace.util.settings' )

_appDir = os.path.dirname( os.path.abspath(sys.argv[0]) )
def getAppDir() :
    return _appDir

def getHomeDir() :
    def valid(path) :
        if path and os.path.isdir(path) :
            return True
        return False
    def env(name) :
        return os.environ.get( name, '' )
    homeDir = env('CSPACEHOMEDIR')
    if os.path.isdir(homeDir):
        return homeDir
    if not valid(homeDir):
        if sys.platform != 'win32' :
            return os.path.expanduser( '~' )
        homeDir = env('USERPROFILE')
        if not valid(homeDir) :
            homeDir = env('HOME')
            if not valid(homeDir) :
                homeDir = '%s%s' % (env('HOMEDRIVE'),env('HOMEPATH'))
                if not valid(homeDir) :
                    homeDir = env('SYSTEMDRIVE')
                    if homeDir and (not homeDir.endswith('\\')) :
                        homeDir += '\\'
                    if not valid(homeDir) :
                        homeDir = 'C:\\'
    return homeDir

def getConfigDir( name ) :
    homeDir = getHomeDir()
    if sys.platform == 'win32' :
        subDir = '_%s' % name
    else :
        subDir = '.%s' % name
    configDir = os.path.join( homeDir, subDir )
    return configDir

class BaseSettings( object ) :
    def __init__( self, configDir ) :
        self.configDir = configDir

    def getConfigDir( self ) :
        return self.configDir

    def _mkdir( self, dpath ) :
        if os.path.isfile(dpath) :
            self._unlink( dpath )
        if not os.path.isdir(dpath) :
            try :
                if sys.platform == 'win32' :
                    os.makedirs( dpath )
                else :
                    os.makedirs( dpath, 0700 )
                return True
            except OSError :
                logger.exception( 'error creating directory: %s', dpath )
                return False
        return True

    def _listdir( self, dpath ) :
        try :
            return os.listdir( dpath )
        except OSError :
            logger.exception( 'error listing dir: %s', dpath )
            return []

    def _unlink( self, fpath ) :
        try :
            if os.path.isdir(fpath) :
                os.rmdir( fpath )
            else :
                os.unlink( fpath )
            return True
        except OSError :
            logger.exception( 'error unlinking: %s', fpath )
            return False

    def _rmrf( self, fpath ) :
        if os.path.isdir(fpath) :
            files = self._listdir( fpath )
            for f in files :
                childPath = os.path.join( fpath, f )
                self._rmrf( childPath )
            return self._unlink( fpath )
        elif os.path.isfile(fpath) :
            return self._unlink( fpath )
        return False

    def _validateElemList( self, elemList ) :
        for x in elemList :
            assert x.strip() == x
            assert '\\' not in x
            assert '/' not in x
            assert x not in ('.','..')

    def _parseEntryPath( self, entryPath ) :
        elemList = entryPath.split( '/' )
        if elemList and (not elemList[0]) :
            del elemList[0]
        if elemList :
            assert elemList[-1]
        self._validateElemList( elemList )
        return elemList

    def _get( self, entryPath ) :
        elemList = self._parseEntryPath( entryPath )
        fpath = os.path.join( self.configDir, *elemList )
        if os.path.isdir(fpath) :
            out = []
            for x in self._listdir(fpath) :
                childPath = '/'.join( elemList + [x] )
                out.append( childPath )
            return out
        elif os.path.isfile(fpath) :
            data = None
            try :
                f = file( fpath, 'rb' )
                data = f.read()
                f.close()
            except (OSError,IOError) :
                logger.exception( 'error reading file: %s', fpath )
            return data
        return None

    def _set( self, entryPath, data ) :
        elemList = self._parseEntryPath( entryPath )
        assert elemList
        dpath = self.configDir
        for d in elemList[:-1] :
            dpath = os.path.join( dpath, d )
            self._mkdir( dpath )
        fpath = os.path.join( dpath, elemList[-1] )
        try :
            f = file( fpath, 'wb' )
            f.write( data )
            f.close()
            return True
        except (OSError,IOError) :
            logger.exception( 'error writing file: %s', fpath )
            return False

    def remove( self, entryPath ) :
        elemList = self._parseEntryPath( entryPath )
        fpath = os.path.join( self.configDir, *elemList )
        return self._rmrf( fpath )

    def listEntries( self, entryPath ) :
        entries = self._get( entryPath )
        t = type(entries)
        if t is StringType :
            logger.warning( 'argument to listEntries is a file: %s', entryPath )
            return []
        if entries is None :
            return []
        assert t is ListType
        return entries

    def getData( self, entryPath, default=None ) :
        data = self._get( entryPath )
        if data is None : return default
        if type(data) is ListType :
            logger.warning( 'argument to getData is a directory: %s', entryPath )
            return default
        return data

    def setData( self, entryPath, value ) :
        return self._set( entryPath, value )

    def getInt( self, entryPath, default=None ) :
        data = self.getData( entryPath )
        if data is None : return default
        try :
            return int(data.strip())
        except ValueError :
            return default

    def setInt( self, entryPath, value ) :
        return self._set( entryPath, str(value) )

    def getString( self, entryPath, default=None ) :
        data = self.getData( entryPath )
        if data is None : return default
        data = data.strip()
        if len(data) < 2 : return default
        if not (data[0] == data[-1] == '"') : return default
        data = data[1:-1]
        try :
            out = []
            i = 0
            while i < len(data) :
                c = data[i]
                if c == '\\' :
                    out.append( data[i+1] )
                    i += 2
                else :
                    out.append( c )
                    i += 1
            return ''.join( out )
        except IndexError :
            return default

    def setString( self, entryPath, value ) :
        out = ['"']
        for x in value :
            if x in '\\"' :
                out.append( '\\' )
            out.append( x )
        out.append( '"' )
        return self._set( entryPath, ''.join(out) )

class LocalSettings( BaseSettings ) :
    def __init__( self, name ) :
        configDir = getConfigDir( name )
        BaseSettings.__init__( self, configDir )
        self._mkdir( configDir )

class AppSettings( BaseSettings ) :
    def __init__( self ) :
        BaseSettings.__init__( self, getAppDir() )



import os, sys, StringIO
from types import StringType
from ncrypt0.rsa import RSAKey, RSAError

from cspace.util.hexcode import hexEncode, hexDecode, HexDecodeError
from cspace.main.common import profileSettings, isValidUserName, LocalSettings

class Contact( object ) :
    def __init__( self, publicKey, name ) :
        assert isValidUserName( name )
        self.publicKey = publicKey
        self.name = name
        self.publicKeyData = publicKey.toDER_PublicKey()

class Profile( object ) :
    def __init__( self, rsaKey, name, keyId, storeEntry ) :
        assert isValidUserName( name )
        self.rsaKey = rsaKey
        self.name = name
        self.keyId = keyId
        self.storeEntry = storeEntry
        self.contactNames = {}
        self.contactKeys = {}

    def addContact( self, contact ) :
        assert contact.publicKeyData not in self.contactKeys
        assert contact.name not in self.contactNames
        self.contactKeys[contact.publicKeyData] = contact
        self.contactNames[contact.name] = contact

    def getContactByName( self, contactName ) :
        return self.contactNames.get( contactName )

    def getContactByPublicKey( self, publicKey ) :
        return self.contactKeys.get( publicKey.toDER_PublicKey() )

    def getContactNames( self ) :
        names = self.contactNames.keys()
        names.sort()
        return names

    def changeContactName( self, oldName, newName ) :
        assert isValidUserName( newName )
        c = self.contactNames.pop( oldName )
        c.name = newName
        self.contactNames[newName] = c

    def removeContact( self, contact ) :
        del self.contactNames[contact.name]
        del self.contactKeys[contact.publicKeyData]

def listProfiles(location='CSpaceProfiles') :
    ps = profileSettings(location)
    profiles = []
    for entry in ps.listEntries('') :
        userName = ps.getData( entry+'/Name' )
        keyId = ps.getData( entry+'/KeyID' )
        profiles.append( (userName,keyId,entry) )
    return profiles

def loadProfile( entry, password, location='CSpaceProfiles') :
    ps = profileSettings(location)
    userName = ps.getData( entry+'/Name' )
    keyId = ps.getData( entry+'/KeyID' )
    encKey = ps.getData( entry+'/PrivateKey' )
    rsaKey = RSAKey()
    try :
        rsaKey.fromPEM_PrivateKey( encKey, password )
    except RSAError :
        return None
    profile = Profile( rsaKey, userName, keyId, entry )
    contactsData = ps.getData( entry+'/ContactList', '' )
    for line in contactsData.split('\n') :
        line = line.strip()
        if not line : continue
        name,hexKey = line.split(':')
        assert isValidUserName(name)
        pubKey = RSAKey()
        pubKey.fromDER_PublicKey( hexDecode(hexKey) )
        contact = Contact( pubKey, name )
        profile.addContact( contact )
    return profile

def createProfile( rsaKey, password, userName, keyId, location='CSpaceProfiles' ) :
    assert isValidUserName( userName )
    ps = profileSettings(location)
    baseEntry = userName
    entry = baseEntry
    suffix = 0
    while ps.getData(entry+'/PrivateKey') :
        suffix += 1
        entry = '%s-%d' % (baseEntry,suffix)
    encKey = rsaKey.toPEM_PrivateKey( password )
    ps.setData( entry+'/PrivateKey', encKey )
    ps.setData( entry+'/Name', userName )
    if keyId is not None :
        ps.setData( entry+'/KeyID', keyId )
    profile = Profile( rsaKey, userName, keyId, entry )
    return profile

def saveProfileContacts( profile, location='CSpaceProfiles' ) :
    out = StringIO.StringIO()
    for name in profile.getContactNames() :
        c = profile.getContactByName( name )
        hexKey = hexEncode( c.publicKeyData )
        print>>out, '%s:%s' % (name,hexKey)
    ps = profileSettings(location)
    ps.setData( profile.storeEntry + '/ContactList', out.getvalue() )

def clearProfileContacts( profile, location='CSpaceProfiles' ):
    ps = profileSettings(location)
    ps.setData( profile.storeEntry + '/ContactList', '' )



import sys

if sys.platform  == 'win32' :
     # no such thing as WSAEPERM or error code 10001 according to winsock.h or MSDN
    EPERM=object()
    from errno import WSAEINVAL as EINVAL
    from errno import WSAEWOULDBLOCK as EWOULDBLOCK
    from errno import WSAEINPROGRESS as EINPROGRESS
    from errno import WSAEALREADY as EALREADY
    from errno import WSAECONNRESET as ECONNRESET
    from errno import WSAEISCONN as EISCONN
    from errno import WSAENOTCONN as ENOTCONN
    from errno import WSAEINTR as EINTR
    from errno import WSAENOBUFS as ENOBUFS
    EAGAIN=EWOULDBLOCK
else:
    from errno import EPERM
    from errno import EINVAL
    from errno import EWOULDBLOCK
    from errno import EINPROGRESS
    from errno import EALREADY
    from errno import ECONNRESET
    from errno import EISCONN
    from errno import ENOTCONN
    from errno import EINTR
    from errno import ENOBUFS
    from errno import EAGAIN

del sys



import os, glob, logging
from tempfile import mkstemp
from nitro.http import HttpRequest
from cspace.main.common import appSettings

logger = logging.getLogger("cspace.main.autoupdater")

def currentBuildNumber() :
    return appSettings().getInt( 'BuildNumber.txt', -1 )

UPDATE_CHECK_INTERVAL = 30*60
UPDATE_BASE_URL = 'http://cspace.in/downloads/'
UPDATE_LATEST_VERSION = 'LatestVersion.txt'

class AutoUpdater( object ) :
    def __init__( self, reactor ) :
        self.reactor = reactor
        self.buildNumber = currentBuildNumber()
        self.timerOp = None
        self.httpOp = None
        self.updateCallback = None
        self._startTimer()

    def setUpdateCallback( self, updateCallback ) :
        self.updateCallback = updateCallback

    def shutdown( self ) :
        if self.timerOp : self.timerOp.cancel()
        if self.httpOp : self.httpOp.cancel()

    def _startTimer( self ) :
        assert self.timerOp is None
        self.timerOp = self.reactor.callLater( UPDATE_CHECK_INTERVAL, self._onTimer )

    def _onTimer( self ) :
        self.deleteOldInstallers()
        self.timerOp = None
        url = UPDATE_BASE_URL + UPDATE_LATEST_VERSION
        http = HttpRequest( self.reactor )
        self.httpOp = http.get( url, self._onGetVersionInfo )

    def _onGetVersionInfo( self, responseCode, data ) :
        global logger
        self.httpOp = None
        if responseCode != 200 :
            logger.error( 'unable to get version info, response = %d' % responseCode )
            self._startTimer()
            return
        installers = []
        try :
            for line in data.split('\n') :
                line = line.strip()
                if not line : continue
                info = line.split( ':' )
                for i,x in enumerate(info) :
                    if i > 0 : info[i] = int(x)
                fileName,fileSize,buildNumber,requires = info
                info = (-buildNumber,fileSize,requires,fileName)
                installers.append( info )
        except (TypeError,ValueError), e :
            logger.error( 'error parsing version info, e =', e )
            self._startTimer()
            return
        installers.sort()
        for info in installers :
            buildNumber,fileSize,requires,fileName = info
            buildNumber = -buildNumber
            if buildNumber <= self.buildNumber :
                self._startTimer()
                return
            if requires > self.buildNumber : continue
            logger.info( 'fetching installer = %s' % fileName )
            url = UPDATE_BASE_URL + fileName
            self.updateFileName = fileName
            http = HttpRequest( self.reactor )
            self.httpOp = http.get( url, self._onFetchInstaller )
            return
        self._startTimer()

    def _onFetchInstaller( self, responseCode, data ) :
        self.httpOp = None
        if responseCode != 200 :
            logger.error( 'unable to fetch update installer, response = %d' % responseCode )
            self._startTimer()
            return
        logger.error( 'fetched update installer: %d' % len(data) )
        if self.updateCallback :
            self.updateCallback( self.updateFileName, data )
            return
        self._startTimer()
        return

    def deleteOldInstallers( self ) :
        global logger
        for envVar in 'TEMP TMP TMPDIR'.split() :
            if not os.environ.has_key(envVar) : continue
            tempDir = os.environ[envVar]
            if not os.path.isdir(tempDir) : continue
            fileList = []
            try :
                fileList = glob.glob( os.path.join(tempDir,'CSpaceUpdate*.exe') )
            except Exception, e :
                logger.error( 'unable to list installers in %s: %s' % (tempDir,e) )
            for f in fileList :
                try :
                    os.unlink( f )
                    logger.info( 'deleted old installer %s' % f )
                except Exception, e :
                    logger.warn( 'unable to delete installer %s: %s' % (f,e) )

    def saveInstaller( self, updateFileName, updateFileData ) :
        for envVar in 'TEMP TMP TMPDIR'.split() :
            if not os.environ.has_key(envVar) : continue
            tempDir = os.environ[envVar]
            if not os.path.isdir(tempDir) : continue
            try :
                fd,filePath = mkstemp( '.exe', 'CSpaceUpdate', tempDir )
            except :
                continue
            try :
                os.close( fd )
                out = file( filePath, 'wb' )
                out.write( updateFileData )
                out.close()
            except :
                continue
            return filePath
        return None



#!/usr/bin/python
#backup_tar.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2013
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#  We want a pipe output or input so we don't need to store intermediate data.
#
#  The popen starts another process.  That process can block but we don't.
#  backup.py only takes data from this pipe when it is ready.

import os
import sys
import subprocess


import lib.nonblocking as nonblocking
import lib.dhnio as dhnio


# Returns file descriptor for process that makes tar archive
def backuptar(directorypath, recursive_subfolders=True):
    if not os.path.isdir(directorypath):
        dhnio.Dprint(1, 'backup_tar.backuptar ERROR %s not found' % directorypath)
        return None

    subdirs = 'subdirs'
    if not recursive_subfolders:
        subdirs = 'nosubdirs'

    dhnio.Dprint(6, "backup_tar.backuptar %s %s" % (directorypath, subdirs))

    if dhnio.Windows():
        if dhnio.isFrozen():
            commandpath = "dhnbackup.exe"
            cmdargs = [commandpath, subdirs, directorypath]
        else:
            commandpath = "dhnbackup.py"
            cmdargs = ['python.exe', commandpath, subdirs, directorypath]
    else:
        commandpath = "dhnbackup.py"
        cmdargs = ['python', commandpath, subdirs, directorypath]

    if not os.path.isfile(commandpath):
        dhnio.Dprint(1, 'backup_tar.backuptar ERROR %s not found' % commandpath)
        return None

    dhnio.Dprint(6, "backup_tar.backuptar going to execute %s" % str(cmdargs))

    try:
        if dhnio.Windows():
            import win32process
            p = nonblocking.Popen(
                cmdargs,
                shell=False,
                stdin=subprocess.PIPE,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                universal_newlines=False,
                creationflags = win32process.CREATE_NO_WINDOW,)
        else:
            p = nonblocking.Popen(
                cmdargs,
                shell=False,
                stdin=subprocess.PIPE,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                universal_newlines=False,)
    except:
        dhnio.Dprint(1, 'backup_tar.backuptar ERROR executing: ' + str(cmdargs) + str(dhnio.formatExceptionInfo()))
        return None

    return p







#!/usr/bin/python
#io_throttle.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2013
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#    When reconstructing a backup we don't want to take over everything
#    and make DHN unresponsive by requesting 1000's of files at once
#    and make it so no other packets can go out,
#    this just tries to limit how much we are sending out or receiving at any time
#    so that we still have control.
#    Before requesting another file or sending another one out
#    I check to see how much stuff I have waiting.  


import os
import sys
import time


try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in io_throttle.py')


import lib.transport_control as transport_control
import lib.dhnio as dhnio
import lib.settings as settings
import lib.dhnpacket as dhnpacket
import lib.commands as commands
import lib.misc as misc
import lib.tmpfile as tmpfile
import lib.nameurl as nameurl


import contact_status

_IO = None
_PacketReportCallbackFunc = None

#------------------------------------------------------------------------------ 

def io():
    global _IO
    if _IO is None:
        _IO = IOThrottle()
    return _IO

def init():
    dhnio.Dprint(4,"io_throttle.init")
    _throttle = io()
    transport_control.AddOutboxPacketStatusFunc(OutboxStatus)

def SetPacketReportCallbackFunc(func):
    global _PacketReportCallbackFunc
    _PacketReportCallbackFunc = func
    
def PacketReport(sendORrequest, supplier_idurl, packetID, result):
    global _PacketReportCallbackFunc
    if _PacketReportCallbackFunc is not None:
        _PacketReportCallbackFunc(sendORrequest, supplier_idurl, packetID, result)

#------------------------------------------------------------------------------ 

def QueueSendFile(fileName, packetID, remoteID, ownerID, callOnAck=None, callOnFail=None):
    return io().QueueSendFile(fileName, packetID, remoteID, ownerID, callOnAck, callOnFail)

def QueueRequestFile(callOnReceived, creatorID, packetID, ownerID, remoteID):
    return io().QueueRequestFile(callOnReceived, creatorID, packetID, ownerID, remoteID)

def DeleteBackupRequests(backupName):
    return io().DeleteBackupRequests(backupName)

def DeleteSuppliers(supplierIdentities):
    return io().DeleteSuppliers(supplierIdentities)

def OutboxStatus(workitem, proto, host, status, error, message):
    return io().OutboxStatus(workitem, proto, host, status, error, message)
    
def IsSendingQueueEmpty():
    return io().IsSendingQueueEmpty()

def HasPacketInSendQueue(supplierIDURL, packetID):
    return io().HasPacketInSendQueue(supplierIDURL, packetID)

def HasBackupIDInSendQueue(supplierIDURL, backupID):
    return io().HasBackupIDInSendingQueue(supplierIDURL, backupID)

def IsBackupSending(backupID):
    return io().IsBackupSending(backupID)

def HasBackupIDInAllQueues(backupID):
    return io().HasBackupIDInAllQueues(backupID)

def OkToSend(supplierIDURL):
    return io().OkToSend(supplierIDURL)

def OkToRequest(supplierIDURL):
    return io().OkToRequest(supplierIDURL)

def GetSendQueueLength(supplierIDURL):
    return io().GetSendQueueLength(supplierIDURL)

def GetRequestQueueLength(supplierIDURL):
    return io().GetRequestQueueLength(supplierIDURL)

#------------------------------------------------------------------------------ 

class FileToRequest:
    def __init__(self, callOnReceived, creatorID, packetID, ownerID, remoteID):
        self.callOnReceived = []
        self.callOnReceived.append(callOnReceived)
        self.creatorID = creatorID
        self.packetID = packetID
        self.ownerID = ownerID
        self.remoteID = remoteID
        self.backupID = packetID[0:packetID.find("-")]
        self.requestTime = None
        self.fileReceivedTime = None
        self.requestTimeout = max(30, 2*int(settings.getBackupBlockSize()/settings.SendingSpeedLimit()))
        self.result = ''
        PacketReport('request', self.remoteID, self.packetID, 'init')
    
    def __del__(self):
        PacketReport('request', self.remoteID, self.packetID, self.result)

#------------------------------------------------------------------------------ 

class FileToSend:
    def __init__(self, fileName, packetID, remoteID, ownerID, callOnAck=None, callOnFail=None):
        self.fileName = fileName
        try:
            self.fileSize = os.path.getsize(os.path.abspath(fileName))
        except:
            dhnio.DprintException()
            self.fileSize = 0
        self.packetID = packetID
        self.remoteID = remoteID
        self.ownerID = ownerID
        self.callOnAck = callOnAck
        self.callOnFail = callOnFail
        self.sendTime = None
        self.ackTime = None
        self.sendTimeout = max( int(self.fileSize/settings.SendingSpeedLimit() ), settings.SendTimeOut() )
        self.result = ''
        PacketReport('send', self.remoteID, self.packetID, 'init')
        
    def __del__(self):
        PacketReport('send', self.remoteID, self.packetID, self.result)

#------------------------------------------------------------------------------ 

#TODO I'm not removing items from the dict's at the moment
class SupplierQueue:
    def __init__(self, supplierIdentity, creatorID):
        self.creatorID = creatorID
        self.remoteID = supplierIdentity
        self.remoteName = nameurl.GetName(self.remoteID)

        # all sends we'll hold on to, only several will be active, 
        # but will hold onto the next ones to be sent
        # self.fileSendQueueMaxLength = 32
        # active files 
        self.fileSendMaxLength = 8  
        # an array of packetId, preserving first in first out, 
        # of which the first maxLength are the "active" sends      
        self.fileSendQueue = []
        # dictionary of FileToSend's using packetId as index, 
        # hold onto stuff sent and acked for some period as a history?         
        self.fileSendDict = {}          

        # all requests we'll hold on to, 
        # only several will be active, but will hold onto the next ones to be sent
        # self.fileRequestQueueMaxLength = 6
        # active requests 
        self.fileRequestMaxLength = 8 
        # an arry of PacketIDs, preserving first in first out
        self.fileRequestQueue = []      
        # FileToRequest's, indexed by PacketIDs
        self.fileRequestDict = {}       

        self.dprintAdjust = 0
        self.shutdown = False

        self.ackedCount = 0
        self.failedCount = 0
        
        self.sendFailedPacketIDs = []
        
        self.sendTask = None
        self.sendTaskDelay = 0.1
        self.requestTask = None
        self.requestTaskDelay = 0.1


    def RemoveSupplierWork(self): 
        # in the case that we're doing work with a supplier who has just been replaced ...
        # Need to remove the register interests
        # our dosend is using acks?
        self.shutdown = True
        for i in range(min(self.fileSendMaxLength, len(self.fileSendQueue))):
            fileToSend = self.fileSendDict[self.fileSendQueue[i]]
            transport_control.RemoveSupplierRequestFromSendQueue(fileToSend.packetID, fileToSend.remoteID, commands.Data())
            transport_control.RemoveInterest(fileToSend.remoteID, fileToSend.packetID)
        for i in range(min(self.fileRequestMaxLength, len(self.fileRequestQueue))):
            fileToRequest = self.fileRequestDict[self.fileRequestQueue[i]]
            transport_control.RemoveSupplierRequestFromSendQueue(fileToRequest.packetID, fileToRequest.remoteID, commands.Retrieve())
            transport_control.RemoveInterest(fileToRequest.remoteID, fileToRequest.packetID)


    def SupplierSendFile(self, fileName, packetID, ownerID, callOnAck=None, callOnFail=None):
        if self.shutdown: 
            dhnio.Dprint(8, "io_throttle.SupplierSendFile finishing to %s, shutdown is True" % self.remoteName)
            return        
        if contact_status.isOffline(self.remoteID):
            dhnio.Dprint(8, "io_throttle.SupplierSendFile %s is offline, so packet %s is failed" % (self.remoteName, packetID))
            if callOnFail is not None:
                reactor.callLater(0, callOnFail, self.remoteID, packetID, 'offline')
            return
        if packetID in self.fileSendQueue:
            # dhnio.Dprint(4, "io_throttle.SupplierSendFile WARNING packet %s already in the queue for %s" % (packetID, self.remoteName))
            if callOnFail is not None:
                reactor.callLater(0, callOnFail, self.remoteID, packetID, 'in queue')
            return
        self.fileSendQueue.append(packetID)
        self.fileSendDict[packetID] = FileToSend(
            fileName, 
            packetID, 
            self.remoteID, 
            ownerID, 
            callOnAck,
            callOnFail,)
        dhnio.Dprint(8, "io_throttle.SupplierSendFile %s to %s, queue=%d" % (packetID, self.remoteName, len(self.fileSendQueue)))
        # reactor.callLater(0, self.DoSend)
        self.DoSend()
            
            
    def RunSend(self):
        #dhnio.Dprint(6, 'io_throttle.RunSend')
        packetsFialed = []
        packetsToRemove = []
        packetsSent = 0
        # let's check all packets in the queue        
        for i in range(len(self.fileSendQueue)):
            packetID = self.fileSendQueue[i]
            fileToSend = self.fileSendDict[packetID]
            # we got notify that this packet was failed to send
            if packetID in self.sendFailedPacketIDs:
                self.sendFailedPacketIDs.remove(packetID)
                packetsFialed.append((packetID, 'failed'))
                continue
            # we already sent the file
            if fileToSend.sendTime is not None:
                packetsSent += 1
                # and we got ack
                if fileToSend.ackTime is not None:
                    # deltaTime = fileToSend.ackTime - fileToSend.sendTime
                    # so remove it from queue
                    packetsToRemove.append(packetID)
                # if we do not get an ack ...    
                else:
                    # ... we do not want to wait to long
                    if time.time() - fileToSend.sendTime > fileToSend.sendTimeout:
                        # so this packet is failed because no response on it 
                        packetsFialed.append((packetID, 'timeout'))
                # we sent this packet already - check next one
                continue
            # the data file to send no longer exists - it is failed situation
            if not os.path.exists(fileToSend.fileName):
                dhnio.Dprint(4, "io_throttle.RunSend WARNING file %s not exist" % (fileToSend.fileName))
                packetsFialed.append((packetID, 'not exist'))
                continue
            # do not send too many packets, need to wait for ack
            # hold other packets in the queue and may be send next time
            if packetsSent > self.fileSendMaxLength:
                # if we sending big file - we want to wait
                # other packets must go without waiting in the queue
                # 10K seems fine, because we need to filter only Data and Parity packets here
                try:
                    if os.path.getsize(fileToSend.fileName) > 1024 * 10:
                        continue
                except:
                    dhnio.DprintException()
                    continue
            # prepare the packet
            Payload = str(dhnio.ReadBinaryFile(fileToSend.fileName))
            newpacket = dhnpacket.dhnpacket(
                commands.Data(), 
                fileToSend.ownerID, 
                self.creatorID, 
                fileToSend.packetID, 
                Payload, 
                fileToSend.remoteID)
            # outbox will not resend, because no ACK, just data, 
            # need to handle resends on own
            transport_control.outboxNoAck(newpacket)  
            transport_control.RegisterInterest(
                self.FileSendAck, 
                fileToSend.remoteID, 
                fileToSend.packetID)
            # mark file as been sent
            fileToSend.sendTime = time.time()
            packetsSent += 1
        # process failed packets
        for packetID, why in packetsFialed:
            self.FileSendFailed(self.fileSendDict[packetID].remoteID, packetID, why)
            packetsToRemove.append(packetID)
        # remove finished packets    
        for packetID in packetsToRemove:
            self.fileSendQueue.remove(packetID)
            del self.fileSendDict[packetID]
        # if sending queue is empty - remove all records about packets failed to send
        if len(self.fileSendQueue) == 0:
            del self.sendFailedPacketIDs[:]
        # remember results
        result = max(len(packetsToRemove), packetsSent)
        # erase temp lists    
        del packetsFialed
        del packetsToRemove
        return result
        

    def SendingTask(self):
        sends = self.RunSend()
        self.sendTaskDelay = misc.LoopAttenuation(
              self.sendTaskDelay, 
              sends > 0, 
              settings.MinimumSendingDelay(), 
              settings.MaximumSendingDelay())
        # attenuation
        self.sendTask = reactor.callLater(self.sendTaskDelay, self.SendingTask)
        
    
    def DoSend(self):
        #dhnio.Dprint(6, 'io_throttle.DoSend')
        if self.sendTask is None:
            self.SendingTask()
        else:
            if self.sendTaskDelay > 1.0:
                self.sendTask.cancel()
                self.sendTask = None
                self.SendingTask()
            

    def FileSendAck(self, packet):    
        if self.shutdown: 
            dhnio.Dprint(8, "io_throttle.FileSendAck finishing to %s, shutdown is True" % self.remoteName)
            return
        self.ackedCount += 1
        if packet.PacketID not in self.fileSendQueue:
            dhnio.Dprint(4, "io_throttle.FileSendAck WARNING packet %s not in sending queue for %s" % (packet.PacketID, self.remoteName))
            return
        if packet.PacketID not in self.fileSendDict.keys():
            dhnio.Dprint(4, "io_throttle.FileSendAck WARNING packet %s not in sending dict for %s" % (packet.PacketID, self.remoteName))
            return
        self.fileSendDict[packet.PacketID].ackTime = time.time()
        if packet.Command == commands.Ack():
            self.fileSendDict[packet.PacketID].result = 'acked'
            if self.fileSendDict[packet.PacketID].callOnAck:
                reactor.callLater(0, self.fileSendDict[packet.PacketID].callOnAck, packet, packet.OwnerID, packet.PacketID)
        elif packet.Command == commands.Fail():
            self.fileSendDict[packet.PacketID].result = 'failed'
            if self.fileSendDict[packet.PacketID].callOnFail:
                reactor.callLater(0, self.fileSendDict[packet.PacketID].callOnFail, packet.CreatorID, packet.PacketID, 'failed')
        dhnio.Dprint(8, "io_throttle.FileSendAck %s from %s, queue=%d" % (str(packet), self.remoteName, len(self.fileSendQueue)))
        self.DoSend()

        
    def FileSendFailed(self, RemoteID, PacketID, why):
        if self.shutdown: 
            dhnio.Dprint(8, "io_throttle.FileSendFailed finishing to %s, shutdown is True" % self.remoteName)
            return
        self.failedCount += 1
        if PacketID not in self.fileSendDict.keys():
            dhnio.Dprint(4, "io_throttle.FileSendFailed WARNING packet %s not in send dict" % PacketID)
            return
        self.fileSendDict[PacketID].result = why
        fileToSend = self.fileSendDict[PacketID]
        assert fileToSend.remoteID == RemoteID
        dhnio.Dprint(8, "io_throttle.FileSendFailed %s to [%s] because %s" % (PacketID, nameurl.GetName(fileToSend.remoteID), why))
        transport_control.RemoveSupplierRequestFromSendQueue(fileToSend.packetID, fileToSend.remoteID, commands.Data())
        transport_control.RemoveInterest(fileToSend.remoteID, fileToSend.packetID)
        if why == 'timeout':
            contact_status.PacketSendingTimeout(RemoteID, PacketID)
        if fileToSend.callOnFail:
            reactor.callLater(0, fileToSend.callOnFail, RemoteID, PacketID, why)
        self.DoSend()


    def SupplierRequestFile(self, callOnReceived, creatorID, packetID, ownerID):
        if self.shutdown: 
            dhnio.Dprint(8, "io_throttle.SupplierRequestFile finishing to %s, shutdown is True" % self.remoteName)
            if callOnReceived:
                reactor.callLater(0, callOnReceived, packetID, 'shutdown')
            return
        if packetID in self.fileRequestQueue:
            # dhnio.Dprint(4, "io_throttle.SupplierRequestFile WARNING packet %s already in the queue for %s" % (packetID, self.remoteName))
            if callOnReceived:
                reactor.callLater(0, callOnReceived, packetID, 'in queue')
            return
        self.fileRequestQueue.append(packetID)
        self.fileRequestDict[packetID] = FileToRequest(
            callOnReceived, creatorID, packetID, ownerID, self.remoteID)
        dhnio.Dprint(8, "io_throttle.SupplierRequestFile for %s from %s, queue length is %d" % (packetID, self.remoteName, len(self.fileRequestQueue)))
        # reactor.callLater(0, self.DoRequest)
        self.DoRequest()


    def RunRequest(self):
        #dhnio.Dprint(6, 'io_throttle.RunRequest')
        packetsToRemove = []
        for i in range(0, min(self.fileRequestMaxLength, len(self.fileRequestQueue))):
            packetID = self.fileRequestQueue[i]
            currentTime = time.time()
            if self.fileRequestDict[packetID].requestTime is not None:
                # the packet were requested
                if self.fileRequestDict[packetID].fileReceivedTime is None:
                    # but no answer yet ...
                    if currentTime - self.fileRequestDict[packetID].requestTime > self.fileRequestDict[packetID].requestTimeout:
                        # and time is out!!!
                        self.fileRequestDict[packetID].report = 'timeout' 
                        packetsToRemove.append(packetID)
                else:
                    # the packet were received (why it is not removed from the queue yet ???)
                    self.fileRequestDict[packetID].result = 'received'
                    packetsToRemove.append(packetID)
            if self.fileRequestDict[packetID].requestTime is None:
                if not os.path.exists(os.path.join(settings.getLocalBackupsDir(), packetID)): 
                    fileRequest = self.fileRequestDict[packetID]
                    dhnio.Dprint(8, "io_throttle.RunRequest for packetID " + fileRequest.packetID)
                    transport_control.RegisterInterest(
                        self.DataReceived, 
                        fileRequest.creatorID, 
                        fileRequest.packetID)
                    newpacket = dhnpacket.dhnpacket(
                        commands.Retrieve(), 
                        fileRequest.ownerID, 
                        fileRequest.creatorID, 
                        fileRequest.packetID, 
                        "", 
                        fileRequest.remoteID)
                    transport_control.outboxNoAck(newpacket)  
                    fileRequest.requestTime = time.time()
                else:
                    # we have the data file, no need to request it
                    self.fileRequestDict[packetID].result = 'exist'
                    packetsToRemove.append(packetID)
        # remember requests results
        result = len(packetsToRemove)
        # remove finished requests
        if len(packetsToRemove) > 0:
            for packetID in packetsToRemove:
                self.fileRequestQueue.remove(packetID)
        del packetsToRemove
        return result


    def RequestTask(self):
        if self.shutdown:
            return
#        if self.RunRequest() > 0:
#            self.requestTaskDelay = 0.1
#        else:
#            if self.requestTaskDelay < 8.0:
#                self.requestTaskDelay *= 2.0
        requests = self.RunRequest()
        self.requestTaskDelay = misc.LoopAttenuation(
             self.requestTaskDelay,
             requests > 0, 
             settings.MinimumReceivingDelay(), 
             settings.MaximumReceivingDelay())
        # attenuation
        self.requestTask = reactor.callLater(self.requestTaskDelay, self.RequestTask)
        
    
    def DoRequest(self):
        #dhnio.Dprint(6, 'io_throttle.DoRequest')
        if self.requestTask is None:
            self.RequestTask()
        else:
            if self.requestTaskDelay > 1.0:
                self.requestTask.cancel()
                self.requestTask = None
                self.RequestTask()


    def DataReceived(self, packet):   
        # we requested some data from a supplier, just received it
        if self.shutdown: 
            # if we're closing down this queue (supplier replaced, don't any anything new)
            return
        if packet.PacketID in self.fileRequestQueue:
            self.fileRequestQueue.remove(packet.PacketID)
        if self.fileRequestDict.has_key(packet.PacketID):
            self.fileRequestDict[packet.PacketID].fileReceivedTime = time.time()
            self.fileRequestDict[packet.PacketID].result = 'received'
            for callBack in self.fileRequestDict[packet.PacketID].callOnReceived:
                callBack(packet, 'received')
        if self.fileRequestDict.has_key(packet.PacketID):
            del self.fileRequestDict[packet.PacketID]
        dhnio.Dprint(8, "io_throttle.DataReceived %s from %s, queue=%d" % (packet, self.remoteName, len(self.fileRequestQueue)))
        self.DoRequest()


    def DeleteBackupRequests(self, backupName):
        if self.shutdown: 
            # if we're closing down this queue 
            # (supplier replaced, don't any anything new)
            return
        packetsToRemove = []
        for packetID in self.fileSendQueue:
            if packetID.find(backupName) == 0:
                self.FileSendFailed(self.fileSendDict[packetID].remoteID, packetID, 'delete request')
                packetsToRemove.append(packetID)
                dhnio.Dprint(8, 'io_throttle.DeleteBackupRequests %s from send queue' % packetID)
        for packetID in packetsToRemove:
            self.fileSendQueue.remove(packetID)
            del self.fileSendDict[packetID]
        packetsToRemove = []
        for packetID in self.fileRequestQueue:
            if packetID.find(backupName) == 0:
                packetsToRemove.append(packetID)
                dhnio.Dprint(8, 'io_throttle.DeleteBackupRequests %s from request queue' % packetID)
        for packetID in packetsToRemove:
            self.fileRequestQueue.remove(packetID)
            del self.fileRequestDict[packetID]
        if len(self.fileRequestQueue) > 0:
            reactor.callLater(0, self.DoRequest)
        if len(self.fileSendQueue) > 0:
            reactor.callLater(0, self.DoSend)
            #self.DoSend()


    def OutboxStatus(self, workitem, proto, host, status, error, message):
        packetID = workitem.packetid
        if status == 'failed' and packetID in self.fileSendQueue:
            self.sendFailedPacketIDs.append(packetID)
            # reactor.callLater(0, self.DoSend)
            self.DoSend()
            

    def HasSendingFiles(self):
        return len(self.fileSendQueue) > 0
    

    def HasRequestedFiles(self):
        return len(self.fileRequestQueue) > 0
            
  
    def OkToSend(self):
        return len(self.fileSendQueue) < self.fileSendMaxLength


    def OkToRequest(self):
        return len(self.fileRequestQueue) < self.fileRequestMaxLength
    
    
    def GetSendQueueLength(self):
        return len(self.fileSendQueue)
    
    
    def GetRequestQueueLength(self):
        return len(self.fileRequestQueue) 
            
#------------------------------------------------------------------------------ 

# all of the backup rebuilds will run their data requests through this 
# so it gets throttled, also to reduce duplicate requests
class IOThrottle:
    def __init__(self):
        self.creatorID = misc.getLocalID()
        self.supplierQueues = {} #
        self.dprintAdjust = 0
        self.paintFunc = None
        

    def SetSupplierQueueCallbackFunc(self, func):
        self.paintFunc = func


    def DeleteSuppliers(self, supplierIdentities):
        for supplierIdentity in supplierIdentities:
            if supplierIdentity:
                if self.supplierQueues.has_key(supplierIdentity):
                    self.supplierQueues[supplierIdentity].RemoveSupplierWork()
                    del self.supplierQueues[supplierIdentity]


    def DeleteBackupRequests(self, backupName):
        #if settings.getDoBackupMonitor() == "Y":
        for supplierIdentity in self.supplierQueues.keys():
            self.supplierQueues[supplierIdentity].DeleteBackupRequests(backupName)


    def QueueSendFile(self, fileName, packetID, remoteID, ownerID, callOnAck=None, callOnFail=None):
        #dhnio.Dprint(8, "io_throttle.QueueSendFile %s to %s" % (packetID, nameurl.GetName(remoteID)))
        if not os.path.exists(fileName):
            dhnio.Dprint(2, "io_throttle.QueueSendFile ERROR %s not exist" % fileName)
            if callOnFail is not None:
                reactor.callLater(.01, callOnFail, remoteID, packetID, 'not exist')
            return
        if remoteID not in self.supplierQueues.keys():
            self.supplierQueues[remoteID] = SupplierQueue(remoteID, self.creatorID)
            dhnio.Dprint(6, "io_throttle.QueueSendFile made a new queue for %s" % nameurl.GetName(remoteID))
        self.supplierQueues[remoteID].SupplierSendFile(
            fileName, packetID, ownerID, callOnAck, callOnFail,)
            
            
    # return result in the callback: callOnReceived(packet or packetID, state)
    # state is: received, exist, in queue, shutdown
    def QueueRequestFile(self, callOnReceived, creatorID, packetID, ownerID, remoteID):
        # make sure that we don't actually already have the file
        #if packetID != settings.BackupInfoFileName():
        if packetID not in [ settings.BackupInfoFileName(), settings.BackupInfoFileNameOld(), settings.BackupInfoEncryptedFileName(), ]:
            filename = os.path.join(settings.getLocalBackupsDir(), packetID)
            if os.path.exists(filename):
                dhnio.Dprint(4, "io_throttle.QueueRequestFile WARNING %s already exist " % filename)
                if callOnReceived:
                    reactor.callLater(0, callOnReceived, packetID, 'exist')
                return
        if remoteID not in self.supplierQueues.keys():
            # made a new queue for this man
            self.supplierQueues[remoteID] = SupplierQueue(remoteID, self.creatorID)
            dhnio.Dprint(6, "io_throttle.QueueRequestFile made a new queue for %s" % nameurl.GetName(remoteID))
        # dhnio.Dprint(8, "io_throttle.QueueRequestFile asking for %s from %s" % (packetID, nameurl.GetName(remoteID)))
        self.supplierQueues[remoteID].SupplierRequestFile(callOnReceived, creatorID, packetID, ownerID)


    def OutboxStatus(self, workitem, proto, host, status, error, message):
#        if settings.getDoBackupMonitor() == "Y":
        for supplierIdentity in self.supplierQueues.keys():
            self.supplierQueues[supplierIdentity].OutboxStatus(workitem, proto, host, status, error, message)


    def IsSendingQueueEmpty(self):
        for idurl in self.supplierQueues.keys():
            if self.supplierQueues[idurl].HasSendingFiles():
                return False
        return True
    
    
    def IsRequestQueueEmpty(self):
        for idurl in self.supplierQueues.keys():
            if not self.supplierQueues[idurl].HasRequestedFiles():
                return False
        return True
    
    
    def HasPacketInSendQueue(self, supplierIDURL, packetID):
        if not self.supplierQueues.has_key(supplierIDURL):
            return False
        return self.supplierQueues[supplierIDURL].fileSendDict.has_key(packetID)
        
    
    def HasBackupIDInSendQueue(self, supplierIDURL, backupID):
        if not self.supplierQueues.has_key(supplierIDURL):
            return False
        for packetID in self.supplierQueues[supplierIDURL].fileSendDict.keys():
            if packetID.startswith(backupID):
                return True
        return False
    
    
    def IsBackupSending(self, backupID):
        for supplierIDURL in self.supplierQueues.keys():
            if self.HasBackupIDInSendQueue(supplierIDURL, backupID):
                return True 
        return False
        
    
    def OkToSend(self, supplierIDURL):
        if not self.supplierQueues.has_key(supplierIDURL):
            # no queue opened to this man, so the queue is ready 
            return True
        return self.supplierQueues[supplierIDURL].OkToSend()
          
    
    def GetRequestQueueLength(self, supplierIDURL):
        if not self.supplierQueues.has_key(supplierIDURL):
            # no queue opened to this man, so length is zero 
            return 0
        return self.supplierQueues[supplierIDURL].GetRequestQueueLength()
        
 
    def GetSendQueueLength(self, supplierIDURL):
        if not self.supplierQueues.has_key(supplierIDURL):
            # no queue opened to this man, so length is zero 
            return 0
        return self.supplierQueues[supplierIDURL].GetSendQueueLength()


#def _debugSending(fileName, packetID, remoteID, ownerID, callOnAck=None, callOnFail=None):
##    import random
##    if random.randint(0, 3) > 1:
##        reactor.callLater(1, callOnAck, None, remoteID, packetID)
##    else:
#    dhnio.Dprint(6, 'io_throttle._debugSending ' + packetID)
#    reactor.callLater(1, callOnFail, remoteID, packetID, 'failed')
        
    
    
    



#money.py

import os
import sys
import string
import StringIO
import time

import lib.dhnio as dhnio
import lib.misc as misc
import lib.settings as settings
import lib.nameurl as nameurl

#import summary


_receiptId = 0

_InboxReceiptCallback = None

#------------------------------------------------------------------------------

def SetInboxReceiptCallback(cb):
    global _InboxReceiptCallback
    _InboxReceiptCallback = cb

def Receipt(tup):
    SaveReceipt(tup)

def SaveReceipt(data):
    try:
        rfilename = settings.getReceiptsDir() + os.sep + data[0] + '.dhnreceipt'
    except:
        return
    src = ''
    for i in data:
        if isinstance(i, float):
            src += '%f\n' % i
        else:
            src += str(i).strip() + '\n'
    dhnio.Dprint(8, 'money.SaveReceipt %s to %s' % (data[0], rfilename))
    if not dhnio._write_data(rfilename, src.strip()):
        dhnio.Dprint(1, 'money.SaveReceipt ERROR during writing to file ' + rfilename)

    SaveBalance(data[6], data[8], data[0])
    currentBal, currentNTBal, maxReceiptId = LoadBalance()
#    if (int(data[0]) == maxReceiptId): # don't set burn rate unless this packet is most recent receipt, not just filling in a missing receipt
#        try:
#            summary.SetBalanceAndBurnRate(float(data[6])+float(data[8]), float(data[5])+float(data[7]))
#        except:
#            dhnio.Dprint(1, 'money.SaveReceipt ERROR setting burn rate or balance for ' + str(data))

def UnpackReceipt(body): # TODO, appears not to be called
    sin = StringIO.StringIO(str(body))
    ReceiptID = sin.readline().strip()
    DateTime = sin.readline().strip()
    Command = sin.readline().strip()
    FromID = sin.readline().strip()
    ToID = sin.readline().strip()
    Amount = sin.readline().strip()
    Balance = sin.readline().strip()
    Report = sin.read().strip()
    return (ReceiptID, DateTime, Command, FromID, ToID, Amount, Balance, Report)

def UnpackReceipt2(body):
    sin = StringIO.StringIO(str(body))
    ReceiptID = misc.receiptIDstr(sin.readline().strip())
    DateTime = sin.readline().strip()
    Command = sin.readline().strip()
    FromID = sin.readline().strip()
    ToID = sin.readline().strip()
    try:
        Amount = float(sin.readline().strip())
        Balance = float(sin.readline().strip())
        AmountNT = float(sin.readline().strip())
        BalanceNT = float(sin.readline().strip())
    except:
        Amount = 0.0
        Balance = 0.0
        AmountNT = 0.0
        BalanceNT = 0.0
    PayRate = sin.readline().strip()
    Report = sin.read().strip()
    return (ReceiptID, DateTime, Command, FromID, ToID, Amount, Balance, AmountNT, BalanceNT, PayRate, Report)

def UnpackReport(report):
    d = {    'suppliers':   {'space': 0.0, 'costs':  0.0, },
             'customers':   {'space': 0.0, 'income': 0.0, },
             'total':       '',
             'text':        '', }
    try:
        for line in report.split('\n'):
            words = line.strip().split(' ')
            if words[0] == 'supplier':
                d['suppliers'][words[1]] = words[2]
            elif words[0] == 'suppliers':
                d['suppliers']['space'] = words[1]
                d['suppliers']['costs'] = words[2]
            elif words[0] == 'customer':
                d['customers'][words[1]] = words[2]
            elif words[0] == 'customers':
                d['customers']['space'] = words[1]
                d['customers']['income'] = words[2]
            elif words[0] == 'total':
                d['total'] += words[1] + '\n'
            else:
                if not line.strip().startswith('end') and not line.strip().endswith('end'):
                    d['text'] += line.strip() + '\n'
    except:
        dhnio.DprintException()
    return d

def LoadReceipt(path):
    fin = open(path, 'r')
    body = fin.read()
    fin.close()
    return UnpackReceipt2(body)

def ReadReceipt( number ):
    path = settings.getReceiptsDir() + os.sep + number + '.dhnreceipt'
    if not os.path.exists(path):
        dhnio.Dprint(1, 'money.ReadReceipt ERROR file not exist ' + path)
        return None
    return LoadReceipt(path)

def ReadAllReceipts():
    l = []
    burnRate = 0.0
    balanceTrans = 0.0
    balanceNonTrans = 0.0

    try:
        current_balance = ''
        filenames = os.listdir(settings.getReceiptsDir())
        filenames.sort()
        for filename in filenames:
            if not filename.endswith('.dhnreceipt'):
                continue

            receipt_id = filename[:-11]
            if len(receipt_id) != 8:
                continue
            try:
                receipt_id = int(receipt_id)
            except:
                continue
            if receipt_id < 0:
                continue
            if receipt_id > 99999999:
                continue

            path = settings.getReceiptsDir() + os.sep + filename
            if not os.path.exists(path):
                continue

            receipt = LoadReceipt(path)
            if receipt is None:
                continue
            if receipt[2] == '' or receipt[3] == '' or receipt[4] == '': # if we don't have a to, from, or command leave out?
                continue

            tupple = (
                receipt[0],
                receipt[2],
                GetTrueAmount(receipt),
                nameurl.GetName(receipt[3]),
                nameurl.GetName(receipt[4]),
                receipt[1])
            l.append(tupple)

            balanceTrans = float(receipt[6])
            balanceNonTrans = float(receipt[8])
            if receipt[2] == 'space':
                burnRate = float(receipt[5]) + float(receipt[7])

    except:
        dhnio.Dprint(1, 'money.ReadAllReceipts unexpected ERROR')
        dhnio.DprintException()

    #def key_func(t):
    #    try:
    #        return int(t[0])
    #    except:
    #        return 0
    #l.sort(key=key_func)

#    if summary.SetBalanceAndBurnRate is not None:
#        summary.SetBalanceAndBurnRate(balanceTrans+balanceNonTrans, burnRate)

    return l


def LoadBalance():
    src = dhnio._read_data(settings.BalanceFile())
    if src is None:
        src = '0.0 0.0 0'
        dhnio._write_data(settings.BalanceFile(), src)
    words = src.split(' ')
    try:
        b = float(words[0])
        b2 = float(words[1])
    except:
        b = 0.0
        b2 = 0.0
    try:
        r = int(words[2])
    except:
        r = 0

    return b, b2, r


def SaveBalance(balance, balancent, receipt_id):
    src = dhnio._read_data(settings.BalanceFile())
    if src is None:
        src = '0.0 0.0 0'

    words = src.split(' ')
    try:
        b = words[0]
        b2 = words[1]
        r = words[2]
    except:
        b = '0.0'
        b2 = '0.0'
        r = '0'
    try:
        b = float(b)
        b2 = float(b2)
        r = int(r)
    except:
        b = 0.0
        b2 = 0.0
        r = 0
    try:
        balanceV = float(balance)
        balancentV = float(balancent)
        receipt_idV = int(receipt_id)
    except:
        balanceV = 0.0
        balancentV = 0.0
        receipt_idV = 0

    if receipt_idV >= r:
        src = '%f %f %d' % (balanceV, balancentV, receipt_idV)

    return dhnio._write_data(settings.BalanceFile(), src)


def SearchMissingReceipts(last_receipt_id=-1):
    dhnio.Dprint(8, 'money.SearchMissingReceipts ' + str(last_receipt_id))

    def try2remove(filepath):
        try:
            os.remove(filepath)
        except:
            dhnio.Dprint(4, 'money.SearchMissingReceipts.try2remove WARNING can not remove ' + filepath)
        dhnio.Dprint(6, 'money.SearchMissingReceipts.try2remove %s removed' % filepath)

    existing_receipts = set()
    max_index = -1
    for filename in os.listdir(settings.getReceiptsDir()):
        filepath = os.path.join(settings.getReceiptsDir(), filename)
        if not filename.endswith('.dhnreceipt'):
            try2remove(filepath)
            continue
        receipt_id = filename[:-11]
        if len(receipt_id) != 8:
            try2remove(filepath)
            continue
        try:
            receipt_id = int(receipt_id)
        except:
            try2remove(filepath)
            continue
        if receipt_id < 0:
            try2remove(filepath)
            continue
        if receipt_id > 99999999:
            try2remove(filepath)
            continue
        if receipt_id > max_index:
            max_index = receipt_id
        existing_receipts.add(receipt_id)

    if last_receipt_id > max_index:
        max_index = last_receipt_id

    r = []
    dhnio.Dprint(8, 'money.SearchMissingReceipts existing_receipts=%s  max_index=%s' % (str(len(existing_receipts)), str(max_index)))

    for i in range(max_index + 1):
        if i not in existing_receipts:
            r.append(misc.receiptIDstr(i))
        if len(r) >= 100:
            break

    if max_index not in existing_receipts:
        r.insert(0, misc.receiptIDstr(max_index))

    existing_receipts.clear()
    return r


def GetTrueAmount(receipt):  # return a float, receipt from UnpackReceipt2 has the trans and nt as floats already
    try:
        return receipt[7] + receipt[5]
    except:
        dhnio.Dprint(1, 'money.GetTrueAmount ERROR with receipt: ' + str(receipt))
        return 0.0

def InboxReceipt(newpacket):
    global _InboxReceiptCallback
    dhnio.Dprint(6, 'money.InboxReceipt ')
    sio = StringIO.StringIO(newpacket.Payload)
    receipt_body = ''
    for line in sio:
        receipt_body += line
        if line.startswith('end'):
            t = UnpackReceipt2(receipt_body.strip())
            receipt_body = ''
            SaveReceipt(t)
    if _InboxReceiptCallback:
        _InboxReceiptCallback(newpacket)
            
    
def main():
    print SearchMissingReceipts()

if __name__ == '__main__':
    main()









#!/usr/bin/python
#dhntester.py

import os
import sys

import time

if __name__ == "__main__":
    dirpath = os.path.dirname(os.path.abspath(sys.argv[0]))
    sys.path.insert(0, os.path.abspath('datahaven'))
    sys.path.insert(0, os.path.abspath(os.path.join(dirpath, '..')))
    sys.path.insert(0, os.path.abspath(os.path.join(dirpath, '..', '..')))

try:
    import lib.dhnio as dhnio
    ##import dhnio.DprintStd, ReadBinaryFile, _read_dict, _dir_remove, SetDebug, LowerPriority
    from lib.dhnpacket import Unserialize
    from lib.nameurl import FilenameUrl
    from lib.settings import init as settings_init
    from lib.settings import CustomersSpaceFile, getCustomersFilesDir, LocalTesterLogFilename
    from lib.contacts import init as contacts_init
    from lib.commands import init as commands_init
except:
    import traceback
    logfile = open('dhntester.log', 'a')
    traceback.print_exc(file=logfile)
    logfile.close()
    sys.exit(2)

# need to make sure the dhntester log is in a directory the user has permissions for,
# such as the customer data directory.  Possibly move to temp directory?
# myoutput = open(LocalTesterLogFilename(), 'w')
# sys.stdout = myoutput
# sys.stderr = myoutput

#-------------------------------------------------------------------------------
#test all packets for each customer.
#check if he use more space than we gave him and if packets is too old.
def SpaceTime():
    dhnio.DprintStd(2, 'dhntester.SpaceTime ' + str(time.strftime("%a, %d %b %Y %H:%M:%S +0000")))
    space = dhnio._read_dict(CustomersSpaceFile())
    if space is None:
        dhnio.DprintStd(2, 'dhntester.SpaceTime ERROR can not read file ' + CustomersSpaceFile())
        return
    customers_dir = getCustomersFilesDir()
    if not os.path.exists(customers_dir):
        dhnio.DprintStd(2, 'dhntester.SpaceTime ERROR customers folder not exist')
        return

    remove_list = {}
    for customer_filename in os.listdir(customers_dir):
        onecustdir = os.path.join(customers_dir, customer_filename)
        if not os.path.isdir(onecustdir):
            remove_list[onecustdir] = 'is not a folder'
            continue
        idurl = FilenameUrl(customer_filename)
        if idurl is None:
            remove_list[onecustdir] = 'wrong folder name'
            continue
        curspace = space.get(idurl, None)
        if curspace is None:
            continue
        try:
            maxspaceV = float(curspace) * 1024 * 1024 #in bytes
        except:
            remove_list[onecustdir] = 'wrong space value'
            continue

        timedict = {}
        sizedict = {}
        for filename in os.listdir(onecustdir):
            path = os.path.join(onecustdir, filename)
            if not os.path.isfile(path):
                continue
            stats = os.stat(path)
            timedict[path] = stats.st_ctime
            sizedict[path] = stats.st_size

        timelist = timedict.keys()
        currentV = 0
        for path in sorted( timelist, key=lambda x:timedict[x], reverse=True ):
            currentV += sizedict.get(path, 0)
            if currentV < maxspaceV:
                continue
            try:
                os.remove(path)
                dhnio.DprintStd(4, 'dhntester.SpaceTime ' + path + ' removed (current %s/ maximum %s)' % (str(currentV), str(maxspaceV)) )
            except:
                dhnio.DprintStd(4, 'dhntester.SpaceTime ERROR removing ' + path)

            time.sleep(0.1)
        del timelist
        timedict.clear()
        sizedict.clear()

    for path in remove_list.keys():
        if not os.path.exists(path):
            continue
        if os.path.isdir(path):
            try:
                dhnio._dir_remove(path)
                dhnio.DprintStd(4, 'dhntester.SpaceTime ' + path + ' removed (%s)' % (remove_list[path]))
            except:
                dhnio.DprintStd(4, 'dhntester.SpaceTime ERROR removing ' + path)
            continue
        if not os.access(path, os.W_OK):
            os.chmod(path, 0600)
        try:
            os.remove(path)
            dhnio.DprintStd(4, 'dhntester.SpaceTime ' + path + ' removed (%s)' % (remove_list[path]))
        except:
            dhnio.DprintStd(4, 'dhntester.SpaceTime ERROR removing ' + path)

    del remove_list

#------------------------------------------------------------------------------
#test packets after list of customers was changed
def UpdateCustomers():
    dhnio.DprintStd(2, 'dhntester.UpdateCustomers ' + str(time.strftime("%a, %d %b %Y %H:%M:%S +0000")))
    space = dhnio._read_dict(CustomersSpaceFile())
    if space is None:
        dhnio.DprintStd(2, 'dhntester.UpdateCustomers ERROR space file can not read' )
        return
    customers_dir = getCustomersFilesDir()
    if not os.path.exists(customers_dir):
        dhnio.DprintStd(2, 'dhntester.UpdateCustomers ERROR customers folder not exist')
        return

    remove_list = {}
    for customer_filename in os.listdir(customers_dir):
        onecustdir = os.path.join(customers_dir, customer_filename)
        if not os.path.isdir(onecustdir):
            remove_list[onecustdir] = 'is not a folder'
            continue
        idurl = FilenameUrl(customer_filename)
        if idurl is None:
            remove_list[onecustdir] = 'wrong folder name'
            continue
        curspace = space.get(idurl, None)
        if curspace is None:
            remove_list[onecustdir] = 'is not a customer'
            continue

    for path in remove_list.keys():
        if not os.path.exists(path):
            continue
        if os.path.isdir(path):
            try:
                dhnio._dir_remove(path)
                dhnio.DprintStd(4, 'dhntester.UpdateCustomers ' + path + ' removed (%s)' % (remove_list[path]))
            except:
                dhnio.DprintStd(4, 'dhntester.UpdateCustomers ERROR removing ' + path)
            continue
        if not os.access(path, os.W_OK):
            os.chmod(path, 0600)
        try:
            os.remove(path)
            dhnio.DprintStd(4, 'dhntester.UpdateCustomers ' + path + ' removed (%s)' % (remove_list[path]))
        except:
            dhnio.DprintStd(4, 'dhntester.UpdateCustomers ERROR removing ' + path)

#------------------------------------------------------------------------------
#check all packets to be valid
def Validate():
    dhnio.DprintStd(2, 'dhntester.Validate ' + str(time.strftime("%a, %d %b %Y %H:%M:%S +0000")))
    contacts_init()
    commands_init()

    customers_dir = getCustomersFilesDir()
    if not os.path.exists(customers_dir):
        return
    for customer_filename in os.listdir(customers_dir):
        onecustdir = os.path.join(customers_dir, customer_filename)
        if not os.path.isdir(onecustdir):
            continue
        for filename in os.listdir(onecustdir):
            path = os.path.join(onecustdir, filename)
            if not os.path.isfile(path):
                continue

            packetsrc = dhnio.ReadBinaryFile(path)
            packet = Unserialize(packetsrc)
            result = packet.Valid()
            packetsrc = ''
            del packet

            if not result:
                try:
                    os.remove(path) # if is is no good it is of no use to anyone
                    dhnio.DprintStd(4, 'dhntester.Validate ' + path + ' removed (invalid packet)')
                except:
                    dhnio.DprintStd(4, 'dhntester.Validate ERROR removing ' + path)

            time.sleep(0.1)

#------------------------------------------------------------------------------

def main():
    if len(sys.argv) < 2:
        return

    settings_init()

    dhnio.OpenLogFile(LocalTesterLogFilename(), True)
        
    dhnio.StdOutRedirectingStart()

    dhnio.LifeBegins()

    dhnio.SetDebug(0)

    dhnio.InstallLocale()

    dhnio.SetDebug(4)

    commands = {
        'update_customers' : UpdateCustomers,
        'validate' : Validate,
        'space_time' : SpaceTime,
    }

    cmd = commands.get(sys.argv[1], None)

    if not cmd:
        dhnio.DprintStd(2, 'dhntester.main ERROR wrong command')
        return

    dhnio.LowerPriority()

    cmd()

    dhnio.StdOutRedirectingStop()
    
    dhnio.CloseLogFile()

#------------------------------------------------------------------------------ 

if __name__ == "__main__":
    main()









#!/usr/bin/python
#bitcoin.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2013
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.

import os

try:
    import bitcoinrpc
    _API_is_installed = True 
except:
    _API_is_installed = False 

try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in bitcoin.py')
from twisted.internet.defer import Deferred, succeed

import dhnio
import misc

#------------------------------------------------------------------------------ 

_Connection = None
_Accounts = {}
_Balance = None

#------------------------------------------------------------------------------ 

def installed():
    return _API_is_installed

def init(username, password, host, port='8332', local=False, configfile=os.path.expanduser('~/.bitcoin/bitcoin.conf')):
    if not installed():
        dhnio.Dprint(4, 'bitcoin.init WARNING module bitcoin-python is not installed, skip.')
        return
    dhnio.Dprint(4, 'bitcoin.init')
    global _Connection
    if _Connection is None:
        try:
            if local:
                if os.path.isfile(configfile):
                    _Connection = bitcoinrpc.connect_to_local(configfile)
                else:
                    dhnio.Dprint(4, 'bitcoin.init ERROR config file %s not exist' % configfile)
            else:
                _Connection = bitcoinrpc.connect_to_remote(username, password, host, port)
        except Exception, e:
            dhnio.Dprint(4, 'bitcoin.init ERROR initializing connection: %s' % str(e))
            return
    dhnio.Dprint(4, 'bitcoin.init connection: %s' % str(_Connection))
    # update()

def shutdown():
    if not installed():
        dhnio.Dprint(4, 'bitcoin.shutdown WARNING module bitcoin-python is not installed, skip.')
        return
    dhnio.Dprint(4, 'bitcoin.shutdown')
    global _Connection
    if _Connection:
        del _Connection
    _Connection = None

def connection():
    if not installed():
        dhnio.Dprint(4, 'bitcoin.connection WARNING module bitcoin-python is not installed, skip.')
        raise Exception, 'module bitcoinrpc is not installed'
    global _Connection
    if _Connection is None:
        dhnio.Dprint(4, 'bitcoin.connection WARNING connection is not initialized, skip.')
        raise Exception, 'connection is not initialized'
    return _Connection

def update(callback=None):
    global _Connection
    if _Connection is None:
        dhnio.Dprint(4, 'bitcoin.update WARNING connection is not initialized, skip.')
        if callback:
            callback(None)            
        return
    def go():
        global _Accounts
        global _Balance
        try:
            info = connection().getinfo() # listaccounts(as_dict=True)
            _Balance = float(info.balance) # long(round(float(info.balance)*1e8))
            dhnio.Dprint(6, 'bitcoin.update ' + str(_Balance))
        except Exception, e:
            dhnio.Dprint(6, 'bitcoin.update ERROR connection to RPC server: ' + str(e))
            _Balance = None
        if callback:
            reactor.callFromThread(callback, _Balance)
    reactor.callInThread(go)
    
def accounts():
    global _Accounts
    return _Accounts

def balance():
    if not installed():
        return 'not available'
    global _Balance
    if _Balance is None:
        return 'disconnected'
    try:
        return '%s BTC' % misc.float2str(_Balance)
    except:
        return 'error'

#------------------------------------------------------------------------------ 

if __name__ == '__main__':
    dhnio.init()
    dhnio.SetDebug(12)
    import sys
    def _x():
        dhnio.Dprint(4, 'balance is: ' + str(balance()))
        reactor.stop()
    reactor.callLater(0, init, sys.argv[1], sys.argv[2], sys.argv[3], sys.argv[4])
    reactor.callLater(3, update, _x)
    reactor.run()
    
    